package com.bwie.auth.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.user.UserRemoteService;
import com.bwie.common.result.Result;
import com.bwie.common.utils.IdUtils;
import com.bwie.common.utils.JwtUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    private final RedisCache redisCache;
    private final HttpServletRequest request;
    private final UserRemoteService userRemoteService;

    public AuthServiceImpl(RedisCache redisCache, HttpServletRequest request, UserRemoteService userRemoteService) {
        this.redisCache = redisCache;
        this.request = request;
        this.userRemoteService = userRemoteService;
    }
    @Override
    public JwtResponse login(LoginRequest loginRequest) {
        Result<User> findtel = userRemoteService.findtel(loginRequest.getUserTel());
        Assert.isTrue(findtel.isSuccess(),"登陆失败");
        User data = findtel.getData();
        Assert.notNull(data,"电话不存在");
        Assert.isTrue(data.getUserPwd().equals(loginRequest.getUserPwd()),"密码错误");
        HashMap<String, Object> map = new HashMap<>();
        String key = IdUtils.genId();
        map.put(JwtConstants.USER_KEY,key);
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+key,data,TokenConstants.EXPIRATION, TimeUnit.SECONDS);


        return JwtResponse.builder()
                .token(token)
                .expirTime(TokenConstants.EXPIRATION)
                .build();
    }

    @Override
    public User info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return cacheObject;
    }
}
