package com.bwie.car.controller;

import com.bwie.car.mapper.CarMapper;
import com.bwie.car.service.CarService;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Car;
import com.bwie.common.domain.Ord;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.Pay;
import com.bwie.common.domain.response.CarShow;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
@ResponseBody
public class CarController {
    private final CarService carService;
    private final RedisCache redisCache;
    private final CarMapper carMapper;
    private final HttpServletRequest request;


    public CarController(CarService carService, RedisCache redisCache, CarMapper carMapper, HttpServletRequest request) {
        this.carService = carService;
        this.redisCache = redisCache;
        this.carMapper = carMapper;
        this.request = request;
    }

    @PostMapping("addcar")
    public Result addcar(@RequestParam Long goodsId) {
        Car findgoods = carService.findgoods(goodsId, info().getUserId());
        if (findgoods == null) {
            Car build = Car.builder()
                    .userId(info().getUserId())
                    .goodsId(goodsId)
                    .goodsNum(1)
                    .build();
            carService.save(build);
        } else {
            findgoods.setGoodsNum(findgoods.getGoodsNum() + 1);
            carMapper.updateById(findgoods);
        }
        return Result.success();
    }

    @GetMapping("show")
    public Result show() {
        List<CarShow> show = carService.show();
        return Result.success(show);
    }

    @PostMapping("delcar")
    public Result delcar(@RequestParam Long carId) {
        carService.delcar(carId);
        return Result.success();
    }

    @PostMapping("updnum")
    public Result updnum(@RequestBody CarShow carShow) {
        carService.addnum(carShow);
        return Result.success();
    }

    @PostMapping("pay")
    public Result pay(@RequestBody Pay pay){
            carService.pay(pay);
            return Result.success();
    }


    public User info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return cacheObject;
    }



}
