package com.bwie.car.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.car.mapper.CarMapper;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Car;
import com.bwie.common.domain.Ord;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.Pay;
import com.bwie.common.domain.response.CarShow;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.ord.OrdRemoteService;
import com.bwie.common.utils.JwtUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CarServiceImpl extends ServiceImpl<CarMapper, Car> implements CarService {

    private final RedisCache redisCache;
    private final CarMapper carMapper;
    private final HttpServletRequest request;
    private final OrdRemoteService ordRemoteService;

    public CarServiceImpl(RedisCache redisCache, CarMapper carMapper, HttpServletRequest request, OrdRemoteService ordRemoteService) {
        this.redisCache = redisCache;
        this.carMapper = carMapper;
        this.request = request;
        this.ordRemoteService = ordRemoteService;
    }

    @Override
    public Car findgoods(Long goodsId, Long userId) {
        LambdaQueryWrapper<Car> carLambdaQueryWrapper = new LambdaQueryWrapper<>();
        carLambdaQueryWrapper.eq(Car::getGoodsId, goodsId);
        carLambdaQueryWrapper.eq(Car::getUserId,userId);
        Car car = carMapper.selectOne(carLambdaQueryWrapper);
        return car;
    }

    @Override
    public List<CarShow> show() {
        List<CarShow> show = carMapper.show(info().getUserId());
        return show;
    }

    @Override
    public void delcar(Long carId) {
        carMapper.deleteById(carId);
    }



    @Override
    public void addnum(CarShow carShow) {
        carMapper.updnum(carShow);
    }

    @Override
    public void pay(Pay pay) {
        ordRemoteService.addord(pay);
        List<Integer> collect = pay.getCarIds().stream().map(c -> {
            String[] split = c.split("-");
            return Integer.valueOf(split[3]);
        }).collect(Collectors.toList());
        carMapper.deleteBatchIds(collect);
    }

    public User info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return cacheObject;
    }
}
