package com.bwie.es.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.common.utils.RandomUtils;
import com.alibaba.nacos.common.utils.UuidUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.domain.Goods;
import com.bwie.common.domain.request.GoodsReq;
import com.bwie.es.mapper.EsMapper;
import com.bwie.es.service.EsService;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.unit.Fuzziness;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class EsServiceImpl extends ServiceImpl<EsMapper, Goods> implements EsService {
    private final EsMapper esMapper;
    private final RestHighLevelClient client;

    public EsServiceImpl(EsMapper esMapper, RestHighLevelClient client) {
        this.esMapper = esMapper;
        this.client = client;
    }

    @Override
    public List<Goods> findall() {
        LambdaQueryWrapper<Goods> goodsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        List<Goods> goods = esMapper.selectList(goodsLambdaQueryWrapper);
        return goods;
    }

    @Override
    public void BatchAdd(List<Goods> gg) {
        BulkRequest bulkRequest = new BulkRequest();
        for (Goods goods : gg) {
            bulkRequest.add(new IndexRequest("goods").id(goods.getGoodsId().toString())
                    .source(JSON.toJSONString(goods), XContentType.JSON));
        }
        try {
            BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
            Assert.isTrue(!bulk.hasFailures(),"同步失败");
            for (Goods goods : gg) {
                goods.setGoodsSync(1);
                esMapper.updateById(goods);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public List<Goods> show(GoodsReq goodsReq) {
        SearchRequest searchRequest = new SearchRequest("goods");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = new BoolQueryBuilder();

        if(goodsReq.getGoodsName()!=null&&!goodsReq.getGoodsName().equals("")){
            boolQueryBuilder.must(QueryBuilders.matchQuery("goodsName",goodsReq.getGoodsName()).fuzziness(Fuzziness.AUTO));
        }
        if(goodsReq.getGoodsBrand()!=null&&!goodsReq.getGoodsBrand().equals("")){
            boolQueryBuilder.must(QueryBuilders.matchQuery("goodsBrand",goodsReq.getGoodsBrand()).fuzziness(Fuzziness.AUTO));
        }
        searchSourceBuilder.query(boolQueryBuilder);


        HighlightBuilder highlightBuilder = new HighlightBuilder();
        highlightBuilder.field("goodsName");
        highlightBuilder.preTags("<span style=color:\"red\">");
        highlightBuilder.postTags("</span>");
        searchSourceBuilder.highlighter(highlightBuilder);
        searchRequest.source(searchSourceBuilder);

        ArrayList<Goods> goods = new ArrayList<>();
        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
            for (SearchHit hit : search.getHits()) {
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                HighlightField name = highlightFields.get("goodsName");
                StringBuffer stringBuffer = new StringBuffer();
                if(name!=null){
                    Text[] fragments = name.getFragments();
                    for (Text fragment : fragments) {
                        stringBuffer.append(fragment.toString());
                    }
                }

                String sourceAsString = hit.getSourceAsString();
                Goods goods1 = JSON.parseObject(sourceAsString, Goods.class);
                if(name!=null){
                    goods1.setGoodsName(stringBuffer.toString());
                }
                goods.add(goods1);
            }


        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return goods;

    }


}
